/**
 * Copyright (C) 2007 4D Soft Ltd.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * See the LICENSE file for the full license
 */

package hu.soft4d.minerva.uml.xmi;

import java.util.Collection;
import java.util.logging.Logger;

import hu.soft4d.minerva.wsdl.Constants;
import hu.soft4d.minerva.xmi.XmiUtil;
import xmitools.xmiparser.XmiNode;

/**
 * @author Zsolt Kiss Gere, 4D Soft Ltd.
 */
class XmiUmlNode
{
    private static Logger log = Logger.getLogger("UmlNode"); 
    
    XmiUmlModel model;
    XmiNode node;
    
    XmiUmlNode(XmiUmlModel model, XmiNode node)
    {
        this.model = model;
        this.node = node;
    }
    
    String name()
    {
        String myname;
        if( node == null )   
            myname = "EMPTY"; 
        else
            myname = node.name();
        
        if( myname.equals("") )  myname = "EMPTY";
        
        return myname;
    }
    
    /**
     * @return null if the tag tagname does not exist
     */
    String getOptionalTag( String stereotype, String tagname ) throws Exception
    {
        return getTag(Constants.SSM_PROFILE, stereotype, tagname);
    }

    /**
     * @return null if the tag tagname does not exist
     */
    String getOptionalTag( String profile, String stereotype, String tagname ) throws Exception
    {
        return getTag(profile, stereotype, tagname);
    }

    String getRequiredTag( String stereotype, String tagname ) throws Exception
    {
        String tag = getTag(Constants.SSM_PROFILE, stereotype, tagname);
        
        if( tag == null )
            throw new Exception("Expected tag attribute '" + tagname + "' not found." +
                                "\n  Node: " + name() + "|eAnnotations|contents");
        else
            return tag;
    }
    
    /**
     * Get the tag value of 'tagname', defined for the given stereotype in the given profile.
     * Return null if not found.  
     * 
     * Example: profile = 'SSMProfile', stereotype = 'JSSM', tagname = 'wsdlUrl'
     * 
     * @throws Exception If no eAnnotations with source='appliedStereotypes' is found
     */
    protected String getTag( String profile, String stereotype, String tagname ) throws Exception
    {
        // Tag values are contained in a child <eAnnotations source="appliedStereotypes">
        XmiNode annotations = node.getChildByTagName("eAnnotations");
        if( annotations == null )
        {
            log.info("Child 'eAnnotations' not found for node " + name());
            return null;
        }
        
        String source = XmiUtil.getAttribute(annotations, "source");
        
        if( source == null || !source.equals("appliedStereotypes") )
        {
            log.info("Expected attribute source='appliedStereotypes' not found" +
                                "\n  Node: " + name());
            return null;
        }
        // Now get all children called <contents>. Each child contains the tags for one stereotype.
        Collection<XmiNode> contentChildren = annotations.collectChildrenByTagName("contents");
        for( XmiNode content: contentChildren )
        {
            String expectedId = profile + "_0:" + profile + "__" + stereotype;
            
            if(! expectedId.equals( XmiUtil.getXmiType(content) ))
                continue;
            
            return XmiUtil.getAttribute(content, tagname);
        }
        
        // No corresponding tag was found. Return null
        return null;
    }
    
    public String toString()
    {
        return "\t" + name();
    }
}
