/**
 * Copyright (C) 2007 4D Soft Ltd.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * See the LICENSE file for the full license
 */

package hu.soft4d.minerva.uml.xmi;

import hu.soft4d.minerva.wsdl.Constants;
import stsimulator.sts.OperationKind;
import xmitools.xmiparser.XmiNode;

/**
 * @author Zsolt Kiss Gere, 4D Soft Ltd.
 */
class XmiUmlOperation extends XmiUmlNode
{
    XmiUmlParameter[] params;
    XmiUmlParameter retParam;
    
    XmiUmlOperation(XmiUmlModel model, XmiNode node)
    {
        super(model, node);
    }

    XmiUmlOperation(XmiUmlModel model, XmiNode node, XmiUmlParameter[] params, XmiUmlParameter retparam)
    {
        this(model, node);
        this.params = params;
        this.retParam = retparam;
    }
    
    OperationKind getOperationKind() throws Exception
    {
        String opkind = getOptionalTag(Constants.PROFILE_SPECIFICATIONS, Constants.STEREOTYPE_OPERATION, "kind");
        //System.out.println("Got op kind: " + opkind );
        
        String prefix = Constants.SSM_OP_KIND;
        
        // TODO MagicDraw 12.5 BUG or FEATURE: operation kind is "" if it does not exist
        if( opkind == null || opkind.equals(""))
            return OperationKind.REQUESTRESPONSE;
        
        else if( opkind.equals(prefix + "REQUEST_RESPONSE") )
            return OperationKind.REQUESTRESPONSE;
        
        else if( opkind.equals(prefix + "ONE_WAY") )
            return OperationKind.ONEWAY;
            
        else if( opkind.equals(prefix + "NOTIFICATION") )
            return OperationKind.NOTIFICATION;
            
        else if( opkind.equals(prefix + "SOLICIT_RESPONSE") )
            return OperationKind.SOLICITRESPONSE;
            
        else throw new Exception("Unknown operation kind '" + opkind + "'" +
                "\n  Expected REQUEST_RESPONSE, ONE_WAY, NOTIFICATION or SOLICIT_RESPONSE");
    }
    
    /**
     * Hmm... this was removed
     */
    String getInputMessageName() throws Exception
    {
        return getOptionalTag(Constants.STEREOTYPE_OPERATION, "inputMessage");
    }

    String getOutputMessageName() throws Exception
    {
        return getOptionalTag(Constants.STEREOTYPE_OPERATION, "outputMessage");
    }

    public String toString()
    {
        StringBuffer sb = new StringBuffer( super.toString() );
        sb.append("\n");
        
        if( params != null )
            for (int i = 0; i < params.length; i++) 
            {
                sb.append("\t\t" + params[i] + "\n");
            }
        
        return sb.toString();
    }
}
