/**
 * Copyright (C) 2007 4D Soft Ltd.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * See the LICENSE file for the full license
 */

package hu.soft4d.minerva.uml.xmi;

import xmitools.xmiparser.XmiNode;

/**
 * @author Zsolt Kiss Gere, 4D Soft Ltd.
 */
public class XmiUmlParameter extends XmiUmlNode
{
    enum ParameterKind
    {
        PRIMITIVE, COMPLEX
    }
    
    String typeXmiId;
    ParameterKind paramKind;
    
    XmiUmlParameter(XmiUmlModel model, XmiNode node)
    {
        super(model, node);
    }
    
    XmiUmlParameter(XmiUmlModel model, XmiNode node, String typeXmiId, ParameterKind kind)
    {
        this(model, node);
        this.typeXmiId = typeXmiId;
        this.paramKind = kind;
    }
    
    boolean hasSimpleType() throws Exception
    {
        if( paramKind == ParameterKind.PRIMITIVE )
            return true;
        
        else if( paramKind == ParameterKind.COMPLEX )
            return false;
        
        throw new Exception("Parameter kind is not defined for " + name());
    }
    
    XmiUmlNode getTypeNode()
    {
        return model.nodes.get(typeXmiId);
    }
    
    String getTypeName()
    {
        return getTypeNode().name();
    }

    public String toString()
    {
        StringBuffer sb = new StringBuffer( super.toString() );
        XmiUmlNode typenode = model.nodes.get( typeXmiId );
        
        if( typenode != null )
            sb.append(" - " + typenode.name());
        
        return sb.toString();
    }
}
