/**
 * Copyright (C) 2007 4D Soft Ltd.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * See the LICENSE file for the full license
 */

package hu.soft4d.minerva.uml.xmi;

import hu.soft4d.minerva.wsdl.Constants;
import hu.soft4d.util.StringUtil;
import xmitools.xmiparser.XmiNode;

/**
 * @author Zsolt Kiss Gere, 4D Soft Ltd.
 */
class XmiUmlStateMachine extends XmiUmlNode
{
    // Tags defined for the stereotype MPSSM
    String wsdlUrl;
    String namespace;
    String service;
    String port;
    
    // The location variables of the SSM are represented as attributes in the UML model
    XmiUmlParameter[] attributes;
    // The callTriggers seems to not be used
    XmiUmlCallTrigger[] callTriggers;
    XmiUmlState[] states;
    XmiUmlTransition[] transitions;
    
    XmiUmlStateMachine(XmiUmlModel model, XmiNode node)
    {
        super(model, node);
    }

    public XmiUmlStateMachine( XmiUmlModel model, XmiNode node, XmiUmlParameter[] attributes, XmiUmlCallTrigger[] calltriggers, 
                            XmiUmlState[] mystates, XmiUmlTransition[] transitions) throws Exception
    {
        this(model, node);
        this.attributes = attributes;
        this.callTriggers = calltriggers;
        this.states = mystates;
        this.transitions = transitions;
        this.wsdlUrl = getRequiredTag(Constants.STEREOTYPE_STATE_MACHINE, "wsdlUrl");
        this.namespace = getOptionalTag(Constants.STEREOTYPE_STATE_MACHINE, "namespace");
        this.service = getRequiredTag(Constants.STEREOTYPE_STATE_MACHINE, "service");
        this.port = getRequiredTag(Constants.STEREOTYPE_STATE_MACHINE, "port");
    }
    
    public String toString()
    {
        StringBuffer sb = new StringBuffer( super.toString() );
        sb.append("\n");
        
        sb.append("\tAttributes:\n");
        StringUtil.arrayToString(sb, attributes);
        
        sb.append("\tStates:\n");
        StringUtil.arrayToString(sb, states);
        
        sb.append("\tTransitions:\n");
        StringUtil.arrayToString(sb, transitions);
        
        return sb.toString();
    }
}
