/**
 * Copyright (C) 2007 4D Soft Ltd.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * See the LICENSE file for the full license
 */

package hu.soft4d.minerva.uml.xmi;

import hu.soft4d.minerva.wsdl.Constants;
import xmitools.xmiparser.XmiNode;

/**
 * @author Zsolt Kiss Gere, 4D Soft Ltd.
 */
class XmiUmlTransition extends XmiUmlNode
{
    enum Kind
    {
        INPUT, OUTPUT, UNOBSERVABLE
    }
    
    String sourceId;
    String targetId;
    String triggerId;
    Kind kind;
    String update;
    XmiUmlCallTrigger trigger;
    UmlGuard guard;
    UmlTransitionEffect effect;
    
    XmiUmlTransition(XmiUmlModel model, XmiNode node)
    {
        super(model, node);
    }

    XmiUmlTransition( XmiUmlModel model, XmiNode node, String sourceid, 
                          String targetid, String triggerid, UmlGuard guard, UmlTransitionEffect effect) throws Exception
    {
        this(model, node);
        this.sourceId = sourceid;
        this.targetId = targetid;
        this.triggerId = triggerid;
        this.guard = guard;
        this.effect = effect;
        this.kind = getKind();
        this.update = getUpdate();
    }
    
    /**
     *  Transition 'kind' is a tag of SSMTransition
     *  
     *  BUG in MagicDraw 12.5!!!
     *  
     *  kind="INPUT" is not written to the XMI export!
     */
    Kind getKind() throws Exception
    {
        // TODO Kind tag is REQUIRED!
        String s = getOptionalTag(Constants.STEREOTYPE_TRANSITION, "transitionKind");
        
        // MagicDraw bug (or feature?)
        if( s == null || s.equals("") )  s = Constants.INPUT2;
        
        Kind kind_ = kindFromString(s);
        
        if( kind_ == null )
            throw new Exception("Unknown transition kind found '" + s + "'" +
                            "\n  Expected " + Constants.INPUT2 + ", " +
                            Constants.OUTPUT2 + " or " + Constants.UNOBSERVABLE2 +
                            "\n  Node '" + name() + "'");
        return kind_;
    }
    
    /**
     *  'Update' is an optional tag of SSMTransition
     * @throws Exception 
     */
    String getUpdate() throws Exception
    {
        String upd = getOptionalTag(Constants.STEREOTYPE_TRANSITION, "update");

        // Append ';' if it does not end with
        if( upd != null && !upd.equals("") && !upd.endsWith(";") )
            return upd + ";";
        else
            return "";
    }

    // Call trigger
    XmiUmlCallTrigger getCallTrigger()
    {
        if( trigger == null )
            if( triggerId != null && ! triggerId.equals("") )
                trigger = (XmiUmlCallTrigger) model.nodes.get(triggerId);
        return trigger;
    }
    
    XmiUmlState getSourceNode() throws Exception
    {
        return (XmiUmlState) model.nodes.get( sourceId );
    }

    XmiUmlState getTargetNode() throws Exception
    {
        return (XmiUmlState) model.nodes.get( targetId );
    }

    public String toString()
    {
        StringBuffer sb = new StringBuffer( super.toString() );
        
        sb.append(" (" + model.nodes.get(sourceId).name() + " -> "
                       + model.nodes.get(targetId).name() + ")");
        
        // Call trigger
        if( triggerId != null && ! triggerId.equals("") )
        {
            XmiUmlCallTrigger trigger = (XmiUmlCallTrigger) model.nodes.get(triggerId);
            if( trigger != null )
                sb.append("\n\t\ttrigger: \n" + trigger);
        }
        
        // Guard
        sb.append("\t\tguard: [" + guard + "]\n");
        
        // Effect
        sb.append("\t\teffect: \n\t\t" + effect + "");
        
        return sb.toString();
    }
    
    public static Kind kindFromString(String s)
    {
        if( s.toUpperCase().equals(Constants.INPUT2)) return Kind.INPUT;
        else if( s.toUpperCase().equals(Constants.OUTPUT2)) return Kind.OUTPUT;
        else if( s.toUpperCase().equals(Constants.UNOBSERVABLE2)) return Kind.UNOBSERVABLE;
        else
            return null;
    }
}













