package hu.soft4d.minerva.wsdl;

import javax.wsdl.Operation;
import javax.wsdl.OperationType;

import org.apache.axis.wsdl.symbolTable.SymbolTable;

import stsimulator.sts.OperationKind;

/**
 * Class representing a WSDL operation.  
 * 
 * @author Zsolt Kiss-Gere, 4DSoft Ltd.
 */
public class JOperation
{
    private SymbolTable symTable;
    private Operation operation;
    
    public JOperation( Operation op, SymbolTable symTable )
    {
        this.operation = op;
        this.symTable = symTable;
    }

    /**
     * @return OperationType.NOTIFICATION, OperationType.ONE_WAY, OperationType.REQUEST_RESPONSE
     *         or OperationType.SOLICIT_RESPONSE
     */
    public OperationType getType()
    {
        return operation.getStyle();
    }

    public Operation getOperation()
    {
        return operation;
    }

    public JMessage getInputMessage() throws Exception
    {
        if (getType() != OperationType.NOTIFICATION)
            return new JMessage( operation.getInput().getMessage(), Constants.IN_MESSAGE, symTable );
        else
            throw new Exception("A NOTIFICATION operation does not have input message");
    }

    public JMessage getOutputMessage() throws Exception
    {
        if (getType() != OperationType.ONE_WAY)
            return new JMessage( operation.getOutput().getMessage(), Constants.OUT_MESSAGE, symTable );
        else
            throw new Exception("A ONE_WAY operation does not have output message");
    }

    public String getName()
    {
        return operation.getName();
    }
    
    public OperationKind getOperationKind()
    {
        OperationType opType = getType();
        
        if( opType == OperationType.REQUEST_RESPONSE )
            return OperationKind.REQUESTRESPONSE;
        
        else if( opType == OperationType.ONE_WAY )
            return OperationKind.ONEWAY;
        
        else if( opType == OperationType.NOTIFICATION )
            return OperationKind.NOTIFICATION;
        
        else if( opType == OperationType.SOLICIT_RESPONSE )
            return OperationKind.SOLICITRESPONSE;
        
        return null;
    }
}

















