package hu.soft4d.minerva.wsdl;

import java.util.Map;

import javax.xml.namespace.QName;

import org.apache.axis.wsdl.symbolTable.DefinedType;
import org.apache.axis.wsdl.symbolTable.SymbolTable;
import org.apache.axis.wsdl.symbolTable.TypeEntry;
import org.apache.xerces.xs.XSModel;

import stsimulator.types.Type;

/**
 * Represents the structure of a WSDL file. Implementations of this interface
 * will parse a WSDL uri and build a memory structure which represents it.
 * 
 * @author Zsolt Kiss-Gere, 4DSoft Ltd.
 */
public interface WSDLModel 
{
    /**
     * Get the WSDL target namespace.
     */
    public String getTargetNamespace() throws Exception;
    
    /**
     * Get all services defined in this WSDL in a map keyed by their qualified name. 
     */
    public Map<QName, JService> getServices();

    /**
     * Get the service given by serviceName as a JService.
     */
    public JService getService(String serviceName) throws Exception;
    
    /**
     * Get all defined types in this WSDL in a map keyed by their qualified name.
     */
    public Map<QName, DefinedType> getAllDefinedTypes();

    /**
     * Get all schemas defined or used by this WSDL in an array of XSModel-s. 
     */
    public XSModel[] getSchemaModels() throws Exception;
    
    /**
     * Get the SSM type corresponding to this qualified type name.
     */
    public Type getCreateSSMType(QName typename) throws Exception;;
    
    /**
     * Get the SSM type corresponding to this TypeEntry.
     */
    public Type getCreateSSMType(TypeEntry typeEntry) throws Exception;
    
    /**
     * Get all SSM types defined in this WSDL, for which a getSSMType() method was already called.
     */
    public Map<QName, Type> getSSMTypes();
    
    /** 
     * Return the symbol table created by parsing the WSDL. For internal use.
     */
    public SymbolTable getSymbolTable();
}
