package hu.soft4d.minerva.wsdl;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.Vector;

import javax.xml.namespace.QName;

import org.apache.axis.wsdl.symbolTable.Element;
import org.apache.axis.wsdl.symbolTable.ElementDecl;
import org.apache.axis.wsdl.symbolTable.TypeEntry;

/**
 * Some utility functions.
 * 
 * @author Zsolt Kiss-Gere, 4DSoft Ltd.
 */
public class WSDLUtil
{
    /**
     * Get the type entry which an element refers to. If the type also refers 
     * another type, then the chain is followed until no type reference is found.
     */
    public static TypeEntry getElementType( Element element )
    {
        TypeEntry type = element;
        TypeEntry eltype = type.getRefType();
        while( eltype != null ) {
            type = eltype;
            eltype = type.getRefType();
        }
        return type;
    }
    
    public static List<JParameter> getContainedParameters( TypeEntry type ) throws Exception
    {
        if( type.isBaseType() || type.isSimpleType() )
            throw new Exception("Complex type expected for " + type.getQName() );
            
        List<JParameter> params = new ArrayList<JParameter>();
        String myName = type.getQName().getLocalPart();
        
        Vector nelems = type.getContainedElements();
        if( nelems != null )
            for (Iterator iter = nelems.iterator(); iter.hasNext();)
            {
                ElementDecl par = (ElementDecl) iter.next();
                // This par is almost good, except that it has a very annoying 'feature':
                // its local name is the form of "requestQuote>r", no clue why..
                String paramName = correctParameterName( par.getQName().getLocalPart(), myName );
                QName typeName = par.getType().getQName();
                params.add( new JParameter(paramName, typeName) );
            }
        // else it is a complex type with no contained elements (array type, for example)

        return params;
    }
    
    private static String correctParameterName( String name, String parentName )
    {
        if( name.startsWith( parentName+">" ) )
            return name.substring( parentName.length()+1 );
        else
            return name;
    }
}
