/**
 * Copyright (C) 2007 4D Soft Ltd.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * See the LICENSE file for the full license
 */

package hu.soft4d.minerva.wsdl.ssm;

import hu.soft4d.minerva.wsdl.Constants;
import hu.soft4d.minerva.wsdl.JInterface;
import hu.soft4d.minerva.wsdl.JMessage;
import hu.soft4d.minerva.wsdl.JOperation;
import hu.soft4d.minerva.wsdl.JParameter;
import hu.soft4d.minerva.wsdl.JPort;
import hu.soft4d.minerva.wsdl.JPortBinding;
import hu.soft4d.minerva.wsdl.JService;
import hu.soft4d.minerva.wsdl.WSDLModel;
import hu.soft4d.minerva.wsdl.WSDLModelFactory;
import hu.soft4d.util.CollectionUtil;
import hu.soft4d.util.Sequence;
import stsimulator.sts.*;
import stsimulator.types.*;

import java.io.PrintStream;
import java.net.URL;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.logging.Logger;

import javax.xml.namespace.QName;

import org.jdom.Element;
import org.jdom.Namespace;


/**
 * THIS CLASS IS DEPRECATED. USE WSDLSSMFactory INSTEAD
 * 
 * Factory class for generating a SymbolicTransitionSystem, if an SSM schema
 * definition URL is given.
 * <p> The intended usage of this class is:
 * <pre>
 *   SSMHandler ssmh = new SSMHandler( ssmSchemaInstanceURL );
 *   SymbolicTransitionSystem ssm = ssmh.getSSM();
 * </pre>
 * The generated ssm is not checked for consistency, neither it has
 * switch parsers attached.
 * 
 * @deprecated  Use WSDLSSMFactory instead!
 * 
 * @author Zsolt Kiss Gere, 4D Soft Ltd.
 */
public class SSMHandler
{
    private static Logger log = Logger.getLogger( SSMHandler.class.getName() );
    
    private URL schemaUrl;
    private URL wsdlUrl;
    private String wsdlUri;
    private WSDLModel wsdl;
    private String serviceName;
    private JService jservice;
    private String portName;
    private JPort jport;
    private Port port;
    private SSMSchemaModel ssmModel;
    // SSM containers
    private Map<String, Location> locationMap = new HashMap<String, Location>();
    private Map<String, InteractionVariable> intvarMap = new HashMap<String, InteractionVariable>();
    private Map<String, Message> messageMap = new HashMap<String, Message>();
    private Map<String, Operation> operationMap = new HashMap<String, Operation>();
    
    public void dumpSSMContainers(PrintStream out)
    {
        out.println("==== DUMP BEGIN ==================================");
        out.println();
        
        CollectionUtil.dumpMap(out, "Location variables", locationMap);
        
        CollectionUtil.dumpMap(out, "Interaction variables", intvarMap);
        
        CollectionUtil.dumpMap(out, "Operations", operationMap);
        
        CollectionUtil.dumpMap(out, "Messages", messageMap);
        
        out.println();
        out.println("==== DUMP END ====================================");
    }
    
    public static SymbolicTransitionSystem generateSSM( URL wsdlURL, URL spssmURL, 
                                String serviceName, String portName) throws Exception
    {
        SSMHandler ssmh = new SSMHandler( wsdlURL, spssmURL, serviceName, portName );
        SymbolicTransitionSystem ssm = ssmh.getSSM();
        //ssmh.dumpSSMContainers(System.out);
        return ssm;
    }
    
    public SSMHandler(URL wsdlURL, URL schemaInstanceURL, String serviceName, String portName) throws Exception
    {
        this.wsdlUrl = wsdlURL;
        this.schemaUrl = schemaInstanceURL;
        this.serviceName = serviceName;
        this.portName = portName;
        
        // Reset counters
        Sequence.reset();
        
        ssmModel = new SSMSchemaModel( schemaInstanceURL, serviceName, portName );
        wsdlUri = wsdlUrl.toString();
        log.info("Got WSDLUrl '" + wsdlUri + "'");
    }

    public SymbolicTransitionSystem getSSM() throws Exception
    {
        SymbolicTransitionSystem ssm = new SymbolicTransitionSystem();
        
        // String targetNamespace = getWSDL().getTargetNamespace();
        
        // The service
        jservice = getWSDL().getService( serviceName );
        jport = jservice.getPort( portName );
        port = new Port( portName );
        
        // Add the service to the ssm
        Service ssmservice = new Service( serviceName );
        ssmservice.addPort( port );
        ssm.addService( ssmservice );

        // The location variables are the global variables
        List<LocationVarEntry> locvarEntries = ssmModel.getLocVarEntries();
        for( LocationVarEntry entry: locvarEntries )
        {
            // Location variables
            LocationVariable locvar = getLocationVar( entry );
            ssm.addVariable( locvar );
        }
        
        // Create the switches and in the meantime populate the location map
        List<SwitchEntry> switchEntries = ssmModel.getSwitchEntries();
        for (SwitchEntry entry: switchEntries )
        {
            // Switches
            Switch sswitch = createSwitch( entry );
            ssm.addSwitch( sswitch );
            
            // Locations
            String start = entry.startLocation;
            String end = entry.targetLocation;
            
            // Store new Location objects if not already there
            if(!locationMap.containsKey(start))
                locationMap.put(start, new Location(start));
            
            if(!locationMap.containsKey(end))
                locationMap.put(end, new Location(end));
            
            // Insert switch - location associations
            Location startLoc = locationMap.get(start);
            startLoc.addOutgoingSwitch( sswitch );
            
            Location endLoc = locationMap.get(end);
            sswitch.setTarget( endLoc );
        }
        
        // Save the locations to the ssm object
        for( Location location: locationMap.values() )
            ssm.addLocation( location );
        
        // Add the type objects to the ssm
        SSMHelper.addTypes(ssm, getWSDL().getSSMTypes());
        
        // Add the interaction variables to the ssm
        SSMHelper.addInteractionVariables(ssm, intvarMap);
        
        // Location variables already added
        
        // Add operations
        SSMHelper.addOperations(ssm, operationMap);
        
        // Add messages (even unobserv!)
        SSMHelper.addMessages(ssm, messageMap);
        
        // Switches and locations were added previously
        // setCurrentLocation
        Location initialLoc = locationMap.get( ssmModel.getInitialState() );
        if( initialLoc == null )
            throw new Exception("Location specified as initial state " + ssmModel.getInitialState() +
                                " not found");
        ssm.setCurrentLocation( initialLoc );
        
        return ssm;
    }

    private JPort getJPort() throws Exception
    {
        if( jport == null )
            throw new Exception("The port has not been initialized"); // should not happen
        
        return jport;
    }

    /**
     * Location variables are the global variables
     */
    private LocationVariable getLocationVar(LocationVarEntry entry) throws Exception
    {
        String name = entry.name;
        // type is supposed to be of form "prefix:name"
        String type = entry.type;
        // The "var" element
        Element parent = entry.parent;
        
        // Get the type namespace
        String prefix;
        String typename;
        String[] sarr = type.split(":",2);
        // sarr.length can be 1 or 2
        if( sarr.length > 1 ) {
            prefix = sarr[0];
            typename = sarr[1];
        }
        else {
            prefix = "";
            typename = type;
        }
        //log.info("Got type: {" + prefix + "}" + typename);
        
        // The type namespace
        Namespace typeNsp = null;
        if( prefix.equals("") )
            typeNsp = parent.getNamespace();  // the default namespace
        else
            typeNsp = parent.getNamespace( prefix );
        
        if( typeNsp == null )
            throw new Exception("Could not resolve namespace for prefix '" + prefix + "'");
        //log.info("The type namespace is " + typeNsp );
        
        QName qname = new QName(typeNsp.getURI(), typename);
        Type t = wsdl.getCreateSSMType( qname );
        
        return new LocationVariable(name, t, t.generateInitialInstance());
    }

    private Switch createSwitch( SwitchEntry entry ) throws Exception
    {
        // First, we need a Message. We get operation, kind from the switch entry
        Message ssmMsg = null;
        
        if( entry.kind.equals(Constants.UNOBSERVABLE) )
        {
            ssmMsg = Message.generateUnobservableMessageForPort(port);
            // Can one unobservable message be used more times in operations?
            messageMap.put("", ssmMsg);
        }
        else {
            // An (operation, kind) identifies a(n ssm) message
            // which could have been already stored
            String msgKey = entry.operationName + "@" + entry.kind;
            
            if( messageMap.containsKey(msgKey) )
                ssmMsg = messageMap.get(msgKey);

            else {
                ssmMsg = createSSMMessage( entry );
                
                // It is necessary to put in a Map first (instead of directly saving in the ssm)
                // also because the same msg type may be used more times in operations  
                messageMap.put(msgKey, ssmMsg);
            }
        }
        
        String restriction = entry.restriction;
        String update = entry.updateMapping;
        
        Switch sswitch = new Switch( ssmMsg, restriction, update );
        return sswitch;
    }
    
    /**
     *  Create an SSM message corresponding to the given switch entry.
     */
    private Message createSSMMessage( SwitchEntry entry ) throws Exception
    {
        JPortBinding binding = getJPort().getBinding();
        // We are interested only in SOAP bindings
        if( ! binding.isSOAPBinding() )
            throw new Exception("Sorry, we are interested only in SOAP bindings");
        
        JInterface intface = binding.getInterface();
        JOperation op = intface.getOperation( entry.operationName );
        
        JMessage msg = null;
        if( entry.kind.equals(Constants.INPUT) )
            msg = op.getInputMessage();
        else if( entry.kind.equals(Constants.OUTPUT) )
            msg = op.getOutputMessage();
        else
            throw new Exception("Unexpected switch kind '" + entry.kind + "'. " +
                    "\n Expected " + Constants.INPUT + " or " + Constants.OUTPUT);
        
        String msgName = msg.getQName().getLocalPart();
        
        int dir = SSMHelper.getDirection( entry.kind );
        MessageKind kind = SSMHelper.getMessageKind( entry.kind );
        
        List<JParameter> msgParams = msg.getParameters(binding, op, dir);
        
        // The SSM parameter list
        ArrayList<InteractionVariable> paramList = new ArrayList<InteractionVariable>();
        
        // For each parameter, create or retrieve an interaction variable and save it
        for( JParameter jpar: msgParams )
        {
            String name = jpar.getName();
            QName typename = jpar.getTypeQName();
            Type type = getWSDL().getCreateSSMType( typename );
            
            InteractionVariable var = getCreateInteractionVariable(name, type);
            paramList.add( var );
        }
        
        Operation oper = getCreateSSMOperation(op);

        Message ssmMsg = new Message( msgName, kind, oper, port, paramList );
        return ssmMsg;
    }

    /**
     *  Check is an ssm operation is stored with the key op.getName(). 
     *  If yes, return it.
     *  Otherwise store a newly created ssm operation
     *  under the key op.getName(), and return it.
     */
    private Operation getCreateSSMOperation( JOperation op )
    {
        String key = op.getName();
        Operation oper = operationMap.get(key);
        if( oper == null )
        {
            oper = SSMHelper.getSSMOperation(op);
            operationMap.put(key, oper);
        }
        return oper;
    }

    /**
     *  If an interaction variable with the given (name, type) exists in the intvarMap,
     *  return it. Otherwise create a new interaction variable, record it in the map 
     *  and return it. 
     */
    private InteractionVariable getCreateInteractionVariable(String name, Type type)
    {
        String key = getVarKey(name, type);
        InteractionVariable var = intvarMap.get(key);
        if( var != null )
            return var;
        else {
            var = new InteractionVariable(name, type);
            intvarMap.put(key, var);
            return var;
        }
    }

    private WSDLModel getWSDL() throws Exception
    {
        if( wsdl == null )
            wsdl = WSDLModelFactory.newWSDLModel( wsdlUri );
        
        return wsdl;
    }
    
    private String getVarKey(String name, Type type)
    {
        StringBuffer sb = new StringBuffer();
//        sb.append( entry.serviceName );
//        sb.append( entry.portName );
//        sb.append( entry.startLocation );
//        sb.append( entry.targetLocation );
//        sb.append( entry.operationName );
//        sb.append( entry.kind );
        sb.append( name );
        sb.append( "@" );
        sb.append( type.getName() );
        
        return sb.toString();
    }
}













